@@ -54,22 +54,33 @@ under the same terms as Perl itself.
            reverse_reference_name - for references and collections, the field
                                     name of the reverse reference
 
-=head2 name
 
- Usage   : $name = $field->name();
- Function: return the name of this field
-
-=cut
+=cut 
 
 package InterMine::Model::Role::Field;
 
 use Moose::Role;
 
 use InterMine::TypeLibrary qw(ClassDescriptor);
+use MooseX::Types::Moose qw(Str);
 use Scalar::Util qw(refaddr);
 
 requires '_get_moose_type';
 
+=head2 name (Str, ro, required)
+
+The name of the descriptor
+
+=cut 
+
+sub name;
+
+has name => (
+    is => 'ro',
+    isa => Str,
+    required => 1,
+);
+
 =head2 field_class
 
  Usage   : my $class = $field->field_class();
@@ -82,6 +93,9 @@ has field_class => (
     is	     => 'rw',
     isa	     => ClassDescriptor,
     weak_ref => 1,
+    handles => {
+        class_name => 'unqualified_name',
+    },
 );
 
 sub _get_moose_options {
@@ -103,6 +117,19 @@ sub _type_is {
     }
 }
 
+=head1 METHODS 
+
+=head2 to_string
+
+The string representation of a field descriptor, ie. its name
+
+=cut 
+
+sub to_string {
+    my $self = shift;
+    return $self->name;
+}
+
 no Moose;
 
 1;
